=begin
名稱：AN_key_roitx(鍵盤軸向旋轉)
作者：星野傳說
日期：2015.01.25
版本：v1.0
版權：星野傳說的博客 http://blog.sina.com.cn/apex168
使用說明：
鍵盤軸向旋轉：移動滑鼠啟動組或元件，↑X旋轉，←X旋轉，→Z旋轉，↓回轉，VCB修變角度值Enter確認
啟動元件後按住SHIFT可將滑鼠移開懸停，滑鼠鍵點擊元件依當前軸向旋轉。
其它說明：
此外掛程式主要用於快速以組件軸心旋轉之用，用於置放元件後快速利用鍵盤來操作旋轉。
啟動外掛程式默認為Y軸，角度45度。



=end
module AN_Hoshino_TOOL
  module HoshinoTOOL
   class AN_key_roitx

  # 在初始化啟動工具
  def activate 
    model=Sketchup.active_model  #------------------------------回傳=>模型中啟動的實例
	model.start_operation("鍵盤旋轉") #--------------------模型啟動運行 外掛程式
    @entities = []
    # 顯示VCB和狀態資訊
    Sketchup::set_status_text("鍵盤旋轉：移動滑鼠啟動組或元件(按住Shift多選)，↑X旋轉，←X旋轉，→Z旋轉，↓回轉，VCB修變角度值Enter確認,Esc退出", SB_PROMPT)
    Sketchup::set_status_text("Z軸，角度：", SB_VCB_LABEL)
    @v = Geom::Vector3d.new(0,0,1)
    # 預設值
	@angle =nil
	angle  =Sketchup.read_default('ankeyroitx', 'angle')
	@angle = angle.to_i unless angle==nil
    @angle = 45 unless @angle
    @add_selection = 0
  end
      #執行旋轉
  def rotate(v_xyz, angle)
    ss= Sketchup.active_model.selection
	if v_xyz=="x"
	     Sketchup::set_status_text("X軸，角度：", SB_VCB_LABEL)
		 @v = Geom::Vector3d.new(1,0,0)
	end
	if v_xyz=="y"
	     Sketchup::set_status_text("Y軸，角度：", SB_VCB_LABEL)
		 @v = Geom::Vector3d.new(0,1,0)
	end
	if v_xyz=="z"
	     Sketchup::set_status_text("Z軸，角度：", SB_VCB_LABEL)
		 @v = Geom::Vector3d.new(0,0,1)
	end
	if v_xyz=="v"
	   angle=-angle
	end
    ss.each do |e|  
      #pt = e.bounds.origin(中心)
      pt = e.transformation.origin#(原點)
      tr = Geom::Transformation.rotation(pt,@v,angle.degrees)
      Sketchup.active_model.active_entities.transform_entities(tr,e)
    end
  end


  # 在滑鼠移動進行選擇
  def onMouseMove(flags, x, y, view)  
    Sketchup::set_status_text("#{@angle}", SB_VCB_VALUE)
    ph = view.pick_helper
    found = ph.do_pick(x,y)
    picked = ph.best_picked 

    if (picked != nil)
      # 不添加邊緣/面孔選擇
      if (!picked.kind_of? Sketchup::Edge and !picked.kind_of? Sketchup::Face)
	Sketchup.active_model.selection.add(picked) if (picked != @entities)
	@entities = picked
      end
    else
      if @add_selection == 0
	Sketchup.active_model.selection.clear
	@entities = nil
      end
    end
  end
  
  # 隨機旋轉(滑鼠左鍵)
  def onLButtonDown(flags, x, y, view) 
    #@angle = rand(360)
    # 更新VCB值
    #Sketchup::set_status_text("#{@angle}", SB_VCB_VALUE)
    rotate(@angle)
  end
  
  # 如果SHIFT被釋放回正常選擇（啟動元件後按住SHIFT可將滑鼠移開懸停）
  def onKeyUp(key, repeat, flags, view)
    @add_selection = 0 if key == CONSTRAIN_MODIFIER_KEY
  end
  
  # 鍵盤操作
  def onKeyDown(key, repeat, flags, view)
    case key
    when VK_UP    then rotate("x", @angle)         #↑鍵	
    when VK_LEFT  then rotate("y", @angle)         #←鍵
    when VK_RIGHT then rotate("z", @angle)         #→鍵
    when VK_DOWN  then rotate("v", @angle)         #↓鍵
   #when COPY_MODIFIER_KEY then change_axis_up  #COPY_MODIFIER_KEY = Ctrl鍵
   #when VK_END then change_axis_down           #End鍵
    when CONSTRAIN_MODIFIER_KEY then @add_selection = 1 if (repeat == 1)#CONSTRAIN MODIFIER KEY 限制修改(Enter)鍵
    end
  end
  
  # 在VCB修改用戶的角度文
  def onUserText(text,view)
    begin
      value = text.to_i
    rescue
      # 錯誤解析文本
      UI.beep
      value = nil
      Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    @angle = value
	Sketchup.write_default('ankeyroitx', 'angle', "#{@angle.to_s}")
  end
  
  def onCancel(flags,view)#====================================================取消
     Sketchup.send_action "selectSelectionTool:"#-----------------------------SketchUp的發送行為選擇選擇工具
	 model.commit_operation  #模型提交操作
     #return nil
  end#-------------------------------------------------------------------------結束
  
 end # 分類結束
 end #Hoshino_TOOL
end #AN_Hoshino_TOOL

=begin
#---------------------------添加功能表項目--------------------------------------
if( not $key_rotix_menu_loaded )
  add_separator_to_menu("Plugins")
  UI.menu("Plugins").add_item("鍵盤旋轉") { Sketchup.active_model.select_tool HoshinoTOOL::AN_key_roitx.new }
end

$key_rotix_menu_loaded = true 
=end
